## Computes required endowments to the stabilizing fund, for a given default
## risk

## Needs to be run after "sims.R"

source("common.R")
source("ma.R")
source("kr.R")

## Si asym = TRUE, ne comptabilise que les moments où le fonds se vide
budgets.sims <- function(sims, sym = TRUE, asym = FALSE)
{
  if (!inherits(sims, "sims"))
    stop("You must provide an argument of class sims.")

  h.years <- attr(sims, "h.years")

  na <- length(years.list)
  nb <- length(r.annual.list)

  Rs <- numeric()

  for (b in 1:nb)
    Rs <- rbind(Rs, (1 + r.annual.list[b])^(0:h.years))

  if (sym)
    budgets.sym <- array(0, c(nb, na, length(sims)))
  if (asym)
    budgets.asym <- array(0, c(nb, na, length(sims)))

  for(i in 1:length(sims))
    {
      for (a in 1:na)
        {
          x <- ma.annual(sims[[i]], years.list[a], 1, diff = TRUE)
          x[1] <- 0

          if (asym)
            x2 <- pmin(x, 0)

          for (b in 1:nb)
            {
              if (sym)
                budgets.sym[b, a, i] <- -min(cumsum(x/Rs[b, ]))
              if (asym)
                budgets.asym[b, a, i] <- -min(cumsum(x2/Rs[b, ]))
            }
        }
    }

  results <- list()

  if (sym)
    {
      dimnames(budgets.sym) <- list(r.annual.list, years.list, NULL)
      results <- c(results, list(budgets.sym = structure(budgets.sym, class = "budgets", fit = attr(sims, "fit"), h.years = attr(sims, "h.years"))))
    }

  if (asym)
    {
      dimnames(budgets.asym) <- list(r.annual.list, years.list, NULL)
      results <- c(results, list(budgets.asym = structure(budgets.asym, class = "budgets", fit = attr(sims, "fit"), h.years = attr(sims, "h.years"))))
    }

  results
}

get.ref.volume.budgets <- function(budgets)
{
  if (!inherits(budgets, "budgets"))
    stop("get.ref.volume: you must provide an argument of class budgets.")
  get.ref.volume.fit(attr(budgets, "fit"))
}

get.ref.price.budgets <- function(budgets)
{
  if (!inherits(budgets, "budgets"))
    stop("get.ref.price: you must provide an argument of class budgets.")
  get.ref.price.fit(attr(budgets, "fit"))
}

## Generates tables 3.3 and 3.4 page 23 of Villemot (2004)
budgets.table.onecomm <- function(budgets, quant = 0.9, ref.price = get.ref.price.budgets(budgets), ref.volume = get.ref.volume.budgets(budgets), latex = FALSE)
{
  if (!inherits(budgets, "budgets"))
    stop("budgets.table.onecomm: you must provide an argument of class budgets.")

  table <- array(0, c(dim(budgets)[1], dim(budgets)[2], 2))
  for (i in 1:dim(budgets)[1])
    for (j in 1:dim(budgets)[2])
      {
        price.quant <- as.numeric(quantile(budgets[i, j, ], quant))
        table[i, j, 1] <- price.quant / ref.price
        table[i, j, 2] <- ref.volume * price.quant
      }

  dimnames(table) <- list(dimnames(budgets)[[1]], dimnames(budgets)[[2]], c("Volume share", "Value"))

  if (latex)
    latex.budgets.table.onecomm(table)
  else
    table
}

latex.budgets.table.onecomm <- function(table)
{
  cat("\\begin{tabular}{|l|", rep("c|", dim(table)[2]), "}", sep = "", fill = TRUE)
  cat("\\hline", fill = TRUE)

  cat("r $\\backslash$ n", dimnames(table)[[2]], sep = " & ")
  cat(" \\\\ \\hline", fill = TRUE)

  for (i in 1:dim(table)[1])
    {
      r <- paste(as.numeric(dimnames(table)[[1]][i]) * 100, "\\%", sep = "")
      cat(r, formatC(table[i, , 1], digits = 2, format = "f"), sep = " & ")
      cat(" \\\\", fill = TRUE)

      cat(" & [")
      cat(round(table[i, , 2]), sep = "] & [")
      cat("] \\\\ \\hline", fill = TRUE)
    }
  cat("\\end{tabular}", fill = TRUE)
}

##Attend une liste d'éléments de classe "budgets" Attention: r et n doivent
## être des strings
budgets.table.multicomm <- function(budgets.list, r = "0.05", n = "5", latex = FALSE)
{
  comms <- sort(names(budgets.list))

  probs <- c(0.5, 0.9, 0.95, 0.99)
  cols <- c("$D^{-1}(0^+)$", "$D(0.50)$", "$D(0.90)$", "$D(0.95)$", "$D(0.99)$", "Upp. Bd.")

  table <- array(0, c(length(budgets.list), length(cols), 2))
  dimnames(table) <- list(comms, cols, c("Volume share", "Value"))

  for (comm in comms)
    {
      dist <- budgets.list[[comm]][r, n, ]
      table[comm, 1, 1] <- ecdf(dist)(0)
      table[comm, 1, 2] <- NA
      table[comm, 2:(length(probs)+1), 1] <- quantile(dist, probs) / get.ref.price.budgets(budgets.list[[comm]])
      table[comm, 2:(length(probs)+1), 2] <- quantile(dist, probs) * get.ref.volume.budgets(budgets.list[[comm]])
      table[comm, length(cols), 1] <- K.R(as.numeric(n), 1, as.numeric(r)) / (1+as.numeric(r))
      table[comm, length(cols), 2] <- table[comm, length(cols), 1] * get.ref.price.budgets(budgets.list[[comm]]) * get.ref.volume.budgets(budgets.list[[comm]])
    }

  if (latex)
    latex.budgets.table.multicomm(table)
  else
    table
}

latex.budgets.table.multicomm <- function(table)
{
  cat("\\begin{tabular}{|l|", rep("c|", dim(table)[2]), "}", sep = "", fill = TRUE)
  cat("\\hline", fill = TRUE)

  cat("Commodity", dimnames(table)[[2]], sep = " & ")
  cat(" \\\\ \\hline", fill = TRUE)

  for (comm in dimnames(table)[[1]])
    {
      cat(comm.names[[comm]], paste(formatC(table[comm, 1, 1] * 100, digits = 1, format = "f"), "\\%", sep = ""),
          formatC(table[comm, 2:dim(table)[2], 1], digits = 2, format = "f"), sep = " & ")
      cat(" \\\\", fill = TRUE)

      cat(" & & [")
      cat(round(table[comm, 2:dim(table)[2], 2]), sep = "] & [")
      cat("] \\\\ \\hline", fill = TRUE)
    }
  cat("\\end{tabular}", fill = TRUE)
}


make.dist.graph <- function(budgets, r = "0.05", n = "5", trunc = 5000, xr = c(0, 3), path = "/tmp")
{
  if (!inherits(budgets, "budgets"))
    stop("Argument must be of class budgets.")

  X11(width = 5.6, height = 4.2)
  par(bg="white", mar = c(2,2,2,2))

  rf <- get.ref.price.budgets(budgets)

  dist <- budgets[r, n, 1:trunc] / rf

  plot(ecdf(dist), main = "", xlab = "", ylab = "", xlim = xr)

  dev.print(device=postscript, file=paste(path, "/CDF-", attr(budgets, "fit")$comm, ".eps", sep=""), horizontal = FALSE, onefile = FALSE, paper = "special")

  dev.off()
}

budgets.sym <- list()
budgets.asym <- list()

for (comm in names(sims))
{
  data(list = sims[[comm]])
  results <- budgets.sims(eval(as.symbol(sims[[comm]])), sym = TRUE, asym = TRUE)
  rm(list = sims[[comm]])
  budgets.sym[[comm]] <- results$budgets.sym
  budgets.asym[[comm]] <- results$budgets.asym
}

save(budgets.sym, file="data/budgets.sym.RData")
save(budgets.asym, file="data/budgets.asym.RData")

## Generates table 3.2 page 22 of Villemot (2004), and table 2 page 12 of
## Cohen, Fally, Villemot (2007)
print("Symmetric fund:")
print(budgets.table.multicomm(budgets.sym))
cat()

## Generates table 3.11 page 31 of Villemot (2004), and table 4 page 13 of
## Cohen, Fally, Villemot (2007)
print("Asymmetric fund:")
print(budgets.table.multicomm(budgets.asym))
