## Common data and functions to all programs

library(tseries)

data(ifs)

comm.list <- c("COTTON.LIVERPOOL", "TOBACCO", "COPPER", "GOLD", "TEA", "ALUMINUM", "BANANAS.LATIN.AMERICA", "COFFEE.UGANDA", "COFFEE.OTHER.MILDS", "RICE.THAILAND..BANGKOK.", "COCOA.BEANS", "SUGAR.EU", "RUBBER.MALAYSIA", "GROUNDNUT.OIL")

## Quantities used in the simulation of the aggregate fund
volumes <- list(BANANAS.LATIN.AMERICA=11.6, COCOA.BEANS=2.818, COFFEE.OTHER.MILDS=77.40, COTTON.LIVERPOOL=70.58, RICE.THAILAND..BANGKOK.=19.9)

## Names of the commodities (in English or in French)
comm.names <- list(BANANAS.LATIN.AMERICA="Bananas", COCOA.BEANS="Cocoa", COFFEE.OTHER.MILDS="Coffee (Arabica)", COTTON.LIVERPOOL="Cotton", RICE.THAILAND..BANGKOK.="Rice")
##comm.names <- list(BANANAS.LATIN.AMERICA="Bananes", COCOA.BEANS="Cacao (en fèves)", COFFEE.OTHER.MILDS="Café (arabica)", COTTON.LIVERPOOL="Coton", RICE.THAILAND..BANGKOK.="Riz")

## Date and month of the price reference
## The month is also used as reference when converting monthly series to annual ones
ref.month <- 7
ref.year <- 2003
ref.price <- as.numeric(window(usppi[[1]], start=c(ref.year, ref.month), end=c(ref.year, ref.month)))

years.list <- c(1, 2, 4, 5, 10)
r.annual.list <- c(0, 0.01, 0.02, 0.03, 0.04, 0.05)

sims <- list(BANANAS.LATIN.AMERICA="sims.bananas", COCOA.BEANS="sims.cocoa", COFFEE.OTHER.MILDS="sims.coffee", COTTON.LIVERPOOL="sims.cotton", RICE.THAILAND..BANGKOK.="sims.rice")

sims.horizon <- 50
sims.length <- 1000
#sims.length <- 100000

get.level.monthly.data <- function(comm)
{
  na.contiguous(commodities[[comm]] / usppi[[1]] * ref.price)
}

# Prends une série mensuelle et renvoie la série (annuelle) des moyennes annuelles
make.annual.ma <- function(s)
{
  if (start(s)[2] == 1)
    year <- start(s)[1]
  else
    year <- start(s)[1] + 1
  r <- window(s, start=c(year, 1), frequency=1)
  for (i in 2:12)
    {
      r2 <- window(s, start=c(year, i), frequency=1)
      tsp(r2) <- c(year, year+length(r2)-1, 1)
      r <- r + r2
    }
  r <- r / 12
  r
}

get.level.annual.data <- function(comm)
{
  make.annual.ma(get.level.monthly.data(comm))
}

get.log.monthly.data <- function(comm)
{
  log(get.level.monthly.data(comm))
}

get.log.annual.data <- function(comm)
{
  log(make.annual.ma(get.level.monthly.data(comm)))
}

print.infos <- function(comm)
{
  d <- get.level.monthly.data(comm)
  cat("Dates: ", start(d)[2], "/", start(d)[1], "-", end(d)[2], "/", end(d)[1], sep="", fill = TRUE)
  cat("Units:", commodities.infos[comm, "units"], fill = TRUE)
  p <- as.numeric(window(d, start=c(ref.year, ref.month), end=c(ref.year, ref.month)))
  cat("Reference price:", p, fill = TRUE)
  cat("Seriescode:", commodities.infos[comm, "seriescode"], fill = TRUE)
  cat("Volume (in billions of US$):", round(volumes[[comm]] * p), fill = TRUE)
}

## Extrait le volume de référence d'une commodité à partir d'un objet "fit" généré par fit.mode
get.ref.volume.fit <- function(fit)
{
  if (!inherits(fit, "fit"))
    stop("get.ref.volume: you must provide an argument of class fit.")
  volumes[[fit$comm]]
}

## Extrait le prix de référence d'une commodité à partir d'un objet "fit" généré par fit.mode
get.ref.price.fit <- function(fit)
{
  if (!inherits(fit, "fit"))
    stop("get.ref.price: you must provide an argument of class fit.")

  if(fit$stat)
    as.numeric(exp(fit$arima$coef["intercept"]))
  else
    fit$ref.val
}

## Calcule la valeur de référence à partir d'une liste de classes "fit" généré par fit.mode
get.ref.value.pool <- function(fits)
{
  ref.value <- 0
  for (i in names(fits))
    ref.value <- ref.value + get.ref.price.fit(fits[[i]]) * get.ref.volume.fit(fits[[i]])
  ref.value
}
