## Computes cost to stabilise prizes
## Produces table 3 page 12 in Cohen, Fally, Villemot (2007)

## Needs to be run after "sims.R"

source("common.R")

source("ma.R")

valuation.sims <- function(sims)
{
  if (!inherits(sims, "sims"))
    stop("You must provide an argument of class sims.")

  h.years <- attr(sims, "h.years")

  na <- length(years.list)
  nb <- length(r.annual.list)

  Rs <- numeric()

  for (b in 1:nb)
    Rs <- rbind(Rs, (1 + r.annual.list[b])^(0:h.years))

  valuation <- array(0, c(nb, na, length(sims)))

  for(i in 1:length(sims))
    {
      for (a in 1:na)
        {
	  x <- ma.annual(sims[[i]], years.list[a], 1, diff = TRUE)
	  x[1] <- 0

          for (b in 1:nb)
            {
	      valuation[b, a, i] <- sum(x/Rs[b, ])
	    }
	}
    }


  dimnames(valuation) <- list(r.annual.list, years.list, NULL)
  structure(valuation, class = "budgets", fit = attr(sims, "fit"), h.years = attr(sims, "h.years"))
}

valuations.onecomm <- function(valuation, ref.price = get.ref.price.budgets(valuation))
{
  if (!inherits(valuation, "budgets"))
    stop("budgets must be of class budgets.")

  results <- array(0, c(dim(valuation)[1], dim(valuation)[2], 2))
  for (i in 1:dim(valuation)[1])
    for (j in 1:dim(valuation)[2])
      {
        results[i, j, 1] <- as.numeric(mean(valuation[i, j, ])) / ref.price
        results[i, j, 2] <- as.numeric(median(valuation[i, j, ])) / ref.price
      }

  dimnames(results) <- list(dimnames(valuation)[[1]], dimnames(valuation)[[2]], c("Mean", "Median"))

  results
}

valuations.multicomm <- function(valuations, r = "0.05", n = "5")
{
  comms <- sort(names(valuations))

  results <- array(0, c(length(comms), 4))
  dimnames(results) <- list(comms, c("Mean", "Median", "Min", "Max"))

  for (comm in comms)
    {
      ref.price <- get.ref.price.budgets(valuations[[comm]])
      results[comm, "Mean"] <- mean(valuations[[comm]][r, n, ]) / ref.price
      results[comm, "Median"] <- median(valuations[[comm]][r, n, ]) / ref.price
      results[comm, "Min"] <- min(valuations[[comm]][r, n, ]) / ref.price
      results[comm, "Max"] <- max(valuations[[comm]][r, n, ]) / ref.price
    }

  results
}

valuations <- list()

for (comm in names(sims))
{
  cat(comm, fill = TRUE)
  data(list = sims[[comm]])
  results <- valuation.sims(eval(as.symbol(sims[[comm]])))
  rm(list = sims[[comm]])
  valuations[[comm]] <- results
}

save(valuations, file="data/valuations.RData")

print(valuations.multicomm(valuations))
